Package: niaidMI
Title: Markov Model Multiple Imputation for NIAID OS
Version: 1.1.0
Date: 2022-02-11
Authors@R: c(
  person(
    family = "Morris",
    given = "Nathan",
    email = "morris_nathan@lilly.com",
    role = c("aut", "cre")),
  person(
    family = "Hu",
    given = "Chaoran",
    email = "hu_chaoran@lilly.com",
    role = c("aut")),
  person(
    family = "Eli Lilly and Company",
    role = "cph"))
Description: The implementation of Markov Model Multiple Imputation
  with the application to COVID-19 scale, NIAID OS.
License: MIT + file LICENSE
Depends: R (>= 3.5.0)
Suggests: testthat
Imports: Rcpp (>= 1.0.3)
LinkingTo: Rcpp
RoxygenNote: 7.1.2
NeedsCompilation: yes
Packaged: 2022-02-11 18:54:17 UTC; c300194
Author: Nathan Morris [aut, cre],
  Chaoran Hu [aut],
  Eli Lilly and Company [cph]
Maintainer: Nathan Morris <morris_nathan@lilly.com>
Repository: CRAN
Date/Publication: 2022-02-11 19:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 23:01:34 UTC; unix
Archs: niaidMI.so.dSYM
