README

* BW_imp_CH.R: Independent implementation of niaidMI package from Chaoran Hu.
    (This file will have a copy in test/testthat/.)

* BW_imp_CH_debug.R: Variant of BW_imp_CH.R with adding debug features,
    such as printing intermediate objects.
      
* impute_debug.R: Variant of niaidMI::impute with adding debug features,
    such as printing intermediate objects.
      
* test-imp-explorer.R: This file try to reveal why tiny difference occure between NM and CH's code.

* test_impute_only.R: Prototype of tests/testthat/test-imp.R.

* tests.R: Prototype of all tests in tests/testthat/.